import {Component, OnInit} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {Event} from '../../models/event';
import {ApiService} from '../../services/api/api.service';
import {Change} from '../../models/change';

@Component({
    selector: 'app-event-detail',
    templateUrl: './event-detail.component.html',
    styleUrls: ['./event-detail.component.css']
})
export class EventDetailComponent implements OnInit {
    id: string = '';
    event: Event = {} as Event;
    latestFullChange: Change = {} as Change;
    isDeleted: boolean = false;
    changeDetails: ChangeDetail[] = [];

    constructor(
        private route: ActivatedRoute,
        private apiService: ApiService
    ) {
    }

    ngOnInit(): void {
        this.id = this.route.snapshot.paramMap.get('id') as string;
        console.log(this.id);

        this.apiService.getEventById(this.id).subscribe(event => {
            this.event = event;

            // Set latest full change
            this.latestFullChange = this.event.changes[this.event.changes.length - 1];
            if (this.latestFullChange.is_deleted) {
                this.isDeleted = true;
                this.latestFullChange = this.event.changes[this.event.changes.length - 2];
            }

            this.generateChangeDetails();
        });
    }

    generateChangeDetails() {
        let changes = this.event.changes;

        if (changes.length < 1) {
            return;
        }

        let firstChange = changes.shift();
        this.changeDetails.push({
            timestamp: firstChange!.change_timestamp,
            summary: 'Created',
            hasDetails: false
        });

        let lastChange = firstChange!;

        for (let change of changes) {
            let cTs = change.change_timestamp;
            let cSummary = '';
            let cOld = '';
            let cNew = '';
            let cDetails = false;
            let appendChange = true;

            if (change.is_deleted) {
                cSummary = 'Deleted';
            } else if (change.new_summary !== lastChange.new_summary) {
                cSummary = 'Name changed';
                cOld = lastChange.new_summary;
                cNew = change.new_summary;
                cDetails = true;
            } else if (change.new_description !== lastChange.new_description) {
                cSummary = 'Description changed';
                cOld = lastChange.new_description;
                cNew = change.new_description;
                cDetails = true;
            } else if (change.new_start !== lastChange.new_start) {
                cSummary = 'Start changed';
                cOld = lastChange.new_start.toString();
                cNew = change.new_start.toString();
                cDetails = true;
            } else if (change.new_end !== lastChange.new_end) {
                cSummary = 'End changed';
                cOld = lastChange.new_end.toString();
                cNew = change.new_end.toString();
                cDetails = true;
            } else if (change.new_location !== lastChange.new_location) {
                cSummary = 'Location changed';
                cOld = lastChange.new_location;
                cNew = change.new_location;
                cDetails = true;
            } else if (change.new_organizer !== lastChange.new_organizer) {
                cSummary = 'Organizer changed';
                cOld = lastChange.new_organizer;
                cNew = change.new_organizer;
                cDetails = true;
            } else if (change.new_categories !== lastChange.new_categories) {
                cSummary = 'Type changed';
                cOld = lastChange.new_categories;
                cNew = change.new_categories;
                cDetails = true;
            } else {
                appendChange = false;
            }

            if(appendChange) {
                this.changeDetails.push({
                    timestamp: cTs,
                    summary: cSummary,
                    oldVersion: cOld,
                    newVersion: cNew,
                    hasDetails: cDetails
                });
            }

            lastChange = change;
        }
    }

    closeTab(): void {
        window.close();
    }
}

export interface ChangeDetail {
    timestamp: Date;
    summary: string;
    hasDetails: boolean;
    oldVersion?: string;
    newVersion?: string;
}
